package cz.muni.pv239.wishlist.ui.mywishlist

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.database.repository.MyWishlistRepository
import cz.muni.pv239.wishlist.databinding.FragmentMyWishlistBinding
import cz.muni.pv239.wishlist.model.mywishlist.MyWishlist
import cz.muni.pv239.wishlist.restclient.MyWishlistService
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.ui.mywishlist.adapters.MyWishlistAdapter
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.NetworkUtils
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.util.Objects.hash

class MyWishlistFragment : Fragment() {

    private lateinit var binding: FragmentMyWishlistBinding
    private val pref: SharedPref by lazy {
        SharedPref(requireContext())
    }
    private var myWishlistList: MutableList<MyWishlist> = mutableListOf()
    private val myWishlistRepository: MyWishlistRepository by lazy {
        MyWishlistRepository(requireContext())
    }
    private val myWishlistService: MyWishlistService by lazy {
        RetrofitUtil.createMyWishlistService()
    }

    private val adapter: MyWishlistAdapter by lazy {
        MyWishlistAdapter(
            ::onClickListenerEdit
        )
    }

    private fun onClickListenerEdit(item: MyWishlist) {
        val action =
            MyWishlistFragmentDirections.actionNavigationMyWishlistToNavigationMyWishlistCreateEdit(adapter.currentList.hashCode())
        action.wishlist = item
        findNavController().navigate(action)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentMyWishlistBinding.inflate(layoutInflater, container, false)

        initRecyclerView()
        initButtons()

        return binding.root
    }

    private fun initButtons() {
        val createButton = binding.fbtMyWishlistAdd
        createButton.setOnClickListener {

            val action =
                MyWishlistFragmentDirections.actionNavigationMyWishlistToNavigationMyWishlistCreateEdit(adapter.currentList.hashCode())
            findNavController().navigate(action)
        }

        val shareButton = binding.fbtMyWishlistShare
        shareButton.setOnClickListener {
            val action = MyWishlistFragmentDirections.actionNavigationMyWishlistToNavigationMyWishlistShare()
            findNavController().navigate(action)
        }

    }

    private fun initRecyclerView() {
        val recyclerView = binding.myWishlistRecyclerView
        recyclerView.layoutManager = LinearLayoutManager(context)
        recyclerView.adapter = adapter
        adapter.submitList(myWishlistList)

        ItemTouchHelper(object : ItemTouchHelper.SimpleCallback(4, ItemTouchHelper.RIGHT) {

            override fun onMove(
                recyclerView: RecyclerView,
                viewHolder: RecyclerView.ViewHolder,
                target: RecyclerView.ViewHolder
            ): Boolean {
                return false
            }

            override fun onSwiped(viewHolder: RecyclerView.ViewHolder, direction: Int) {
                val deletedWishlist: MyWishlist = adapter.currentList[viewHolder.adapterPosition]
                val position = viewHolder.adapterPosition

                delete(deletedWishlist.idServer, position) {
                    myWishlistRepository.delete(deletedWishlist)
                    myWishlistList.removeAt(position)
                    adapter.notifyItemRemoved(position)
                    Snackbar.make(
                        recyclerView,
                        "Deleted " + deletedWishlist.name,
                        Snackbar.LENGTH_LONG
                    )
                        .setAction(
                            "Undo"
                        ) {

                            deletedWishlist.idServer = ""
                            saveOrUpdate(deletedWishlist, position, false)

                        }.show()
                }
            }
        }).attachToRecyclerView(recyclerView)
    }

    private fun refreshData() {
        myWishlistList.clear()
        myWishlistList.addAll(myWishlistRepository.getAll())
        adapter.notifyDataSetChanged()
    }

    override fun onResume() {
        getAllMyWishlistFromServer()
        (requireActivity() as MainActivity).setRefreshMyWishlistData(::getAllMyWishlistFromServer)
        super.onResume()
    }

    private fun delete(idServer: String, position: Int, actionAfterSuccess: () -> Unit) {

        if (!canBeDataEdited()) {
            adapter.notifyItemChanged(position)
            return
        }

        changeVisibilityLoadingScreen(true)

        myWishlistService.deleteMyWishlist(
            idServer,
            pref.getInt(Constants.CHECK_SUM_MYWIHSLIST),
            pref.getString(Constants.USER_TOKEN)
        )
            .enqueue(object : Callback<Void> {
                override fun onResponse(
                    call: Call<Void>,
                    response: Response<Void>
                ) {

                    if (response.isSuccessful) {

                        response.headers()["Control-Sum"]?.let {
                            pref.saveInt(Constants.CHECK_SUM_MYWIHSLIST, Integer.parseInt(it))
                        }

                        actionAfterSuccess()
                        changeVisibilityLoadingScreen(false)

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("myWishlist", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }

                }

                override fun onFailure(call: Call<Void>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun getAllMyWishlistFromServer() {

        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            refreshData()
            return
        }

        changeVisibilityLoadingScreen(true)

        myWishlistService.getAllMyWishlist(pref.getString(Constants.USER_TOKEN))
            .enqueue(object : Callback<List<MyWishlist>> {
                override fun onResponse(
                    call: Call<List<MyWishlist>>,
                    response: Response<List<MyWishlist>>
                ) {

                    val listOFWishlists: List<MyWishlist>? = response.body()
                    if (response.isSuccessful && listOFWishlists != null) {

                        response.headers()["Control-Sum"]?.let {
                            pref.saveInt(Constants.CHECK_SUM_MYWIHSLIST, Integer.parseInt(it))
                        }

                        myWishlistRepository.deleteAll()
                        myWishlistRepository.saveOrUpdate(listOFWishlists)

                        refreshData()
                        showMessage("MyWishlist is in sync")
                        changeVisibilityLoadingScreen(false)

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("myWishlist", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        refreshData()
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }

                }

                override fun onFailure(call: Call<List<MyWishlist>>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    refreshData()
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun saveOrUpdate(myWishlist: MyWishlist, position: Int, updating: Boolean) {

        if (!canBeDataEdited()) {
            return
        }

        changeVisibilityLoadingScreen(true)

        myWishlistService.saveOrUpdateMyWishlist(
            myWishlist,
            pref.getInt(Constants.CHECK_SUM_MYWIHSLIST),
            pref.getString(Constants.USER_TOKEN))
                .enqueue(object : Callback<MyWishlist> {
                    override fun onResponse(
                        call: Call<MyWishlist>,
                        response: Response<MyWishlist>
                    ) {

                        val responseBody = response.body()
                        if (response.isSuccessful && responseBody != null) {

                            response.headers()["Control-Sum"]?.let {
                                pref.saveInt(Constants.CHECK_SUM_MYWIHSLIST, Integer.parseInt(it))
                            }

                            if (updating) {
                                responseBody.idMobile = myWishlist.idMobile
                            }

                            myWishlistRepository.saveOrUpdate(responseBody)

                            if (updating) {
                                myWishlistList.removeAt(position)
                                myWishlistList.add(position, responseBody)
                                adapter.notifyItemChanged(position)
                                showMessage("MyWishlist updated")
                            } else {
                                myWishlistList.add(position, responseBody)
                                adapter.notifyItemInserted(position)
                            }

                            changeVisibilityLoadingScreen(false)

                        } else {
                            val errorMessage = RetrofitUtil.parseErrorCode("myWishlist", response.code())
                            Log.e(this::class.simpleName, errorMessage)
                            changeVisibilityLoadingScreen(false)
                            showMessage(errorMessage)
                        }

                    }

                    override fun onFailure(call: Call<MyWishlist>, t: Throwable) {
                        val errorMessage = RetrofitUtil.parseErrorMessage(t)
                        Log.e(this::class.simpleName, errorMessage, t)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }
                })
    }

    private fun canBeDataEdited(): Boolean {
        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            return false
        }

        return true
    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }
}