package cz.muni.pv239.wishlist.ui.sharedwishlists

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.navArgs
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.database.repository.SharedWishlistRepository
import cz.muni.pv239.wishlist.database.repository.mappers.SharedItemRepository
import cz.muni.pv239.wishlist.databinding.FragmentSharedDetailBinding
import cz.muni.pv239.wishlist.model.sharedwishlist.SharedItem
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.SharedItemService
import cz.muni.pv239.wishlist.ui.sharedwishlists.adapters.SharedDetailAdapter
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.NetworkUtils
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class SharedDetailFragment : Fragment() {
    private lateinit var binding: FragmentSharedDetailBinding
    private val wishlistItems: MutableList<SharedItem> = mutableListOf()
    private val sharedWishlistRepository: SharedWishlistRepository by lazy {
        SharedWishlistRepository(requireContext())
    }
    private val sharedItemRepository: SharedItemRepository by lazy {
        SharedItemRepository(requireContext())
    }
    private val args: SharedDetailFragmentArgs by navArgs()
    private val adapter: SharedDetailAdapter by lazy {
        SharedDetailAdapter(
            ::onClickListenerIsClaimed
        )
    }
    private lateinit var pref: SharedPref
    private val sharedItemService: SharedItemService by lazy {
        RetrofitUtil.createSharedItemService()
    }

    private fun onClickListenerIsClaimed(item: SharedItem, position: Int) {
        update(item, position)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View {
        binding = FragmentSharedDetailBinding.inflate(layoutInflater, container, false)
        pref = SharedPref(requireContext())
        initRecyclerView()
        return binding.root
    }

    private fun initRecyclerView() {
        val recyclerView = binding.sharedDetailRecyclerView
        recyclerView.layoutManager = LinearLayoutManager(context)
        recyclerView.adapter = adapter
        adapter.submitList(wishlistItems)
    }

    private fun refreshData() {
        wishlistItems.clear()
        wishlistItems.addAll(sharedWishlistRepository.getByIdWithItems(args.wishlistId).items)
        adapter.notifyDataSetChanged()
    }

    override fun onResume() {
        refreshData()
        super.onResume()
    }

    private fun update(item: SharedItem, position: Int) {
        if (!canBeDataEdited()) {
            return
        }
        changeVisibilityLoadingScreen(true)
        sharedItemService.update(item, args.controlSum, pref.getString(Constants.USER_TOKEN))
            .enqueue(object : Callback<SharedItem> {
                override fun onResponse(
                    call: Call<SharedItem>,
                    response: Response<SharedItem>
                ) {
                    var responseBody = response.body()
                    if (response.isSuccessful && responseBody != null) {

                        responseBody.idMobile = item.idMobile
                        sharedItemRepository.update(responseBody)

                        wishlistItems.removeAt(position)
                        wishlistItems.add(position, responseBody)
                        adapter.notifyItemChanged(position)
                        changeVisibilityLoadingScreen(false)
                        showMessage("Item was updated.")
                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("SharedWishlist", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }
                }

                override fun onFailure(call: Call<SharedItem>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun canBeDataEdited(): Boolean {
        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            return false
        }
        return true
    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }
}