package cz.muni.pv239.wishlist.ui.sharedwishlists

import android.app.AlertDialog
import android.content.DialogInterface
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.ItemTouchHelper
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.snackbar.Snackbar
import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.database.repository.SharedWishlistRepository
import cz.muni.pv239.wishlist.databinding.FragmentSharedWishlistsBinding
import cz.muni.pv239.wishlist.model.mywishlist.MyWishlist
import cz.muni.pv239.wishlist.model.sharedwishlist.SharedItem
import cz.muni.pv239.wishlist.model.sharedwishlist.SharedWishlist
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.SharedItemService
import cz.muni.pv239.wishlist.ui.sharedwishlists.adapters.SharedWishlistsAdapter
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.NetworkUtils
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class SharedWishlistsFragment : Fragment() {
    private lateinit var binding: FragmentSharedWishlistsBinding
    private val sharedWishlistList: MutableList<SharedWishlist> = mutableListOf()
    private val sharedWishlistRepository: SharedWishlistRepository by lazy {
        SharedWishlistRepository(requireContext())
    }
    private val sharedItemService: SharedItemService by lazy {
        RetrofitUtil.createSharedItemService()
    }
    private val pref: SharedPref by lazy {
        SharedPref(requireContext())
    }
    private val adapter: SharedWishlistsAdapter by lazy {
        SharedWishlistsAdapter(
            ::onClickListenerDetail
        )
    }

    private fun onClickListenerDetail(id: Long, checkSum: Int) {
        val action =
            SharedWishlistsFragmentDirections.actionNavigationSharedWishlistsToSharedDetailFragment(
                id,
                checkSum
            )
        findNavController().navigate(action)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSharedWishlistsBinding.inflate(layoutInflater, container, false)

        initElems()
        initRecyclerView()

        return binding.root
    }

    private fun initElems() {
        binding.fbtSharedWishlistAdd.setOnClickListener {
            addSharedWishlist()
        }
    }

    private fun initRecyclerView() {
        val recyclerView = binding.sharedWishlistRecyclerView
        recyclerView.layoutManager = LinearLayoutManager(context)
        recyclerView.adapter = adapter
        adapter.submitList(sharedWishlistList)

        ItemTouchHelper(object : ItemTouchHelper.SimpleCallback(4, ItemTouchHelper.RIGHT) {
            override fun onMove(
                recyclerView: RecyclerView,
                viewHolder: RecyclerView.ViewHolder,
                target: RecyclerView.ViewHolder
            ): Boolean {
                return false
            }

            override fun onSwiped(viewHolder: RecyclerView.ViewHolder, direction: Int) {
                val deletedWishlist: SharedWishlist =
                    adapter.currentList[viewHolder.adapterPosition]
                val position = viewHolder.adapterPosition


                removeSharedWishlist(deletedWishlist.idServer, position) {
                    sharedWishlistRepository.delete(deletedWishlist)
                    sharedWishlistList.removeAt(position)
                    adapter.notifyItemRemoved(position)
                    showMessage("Shared wishlist removed.")

                }
            }
        }).attachToRecyclerView(recyclerView)
    }

    private fun removeSharedWishlist(idServer: String,
                                     position: Int,
                                     actionAfterSuccess: () -> Unit) {

        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            adapter.notifyItemChanged(position)
            return
        }

        changeVisibilityLoadingScreen(true)

        sharedItemService.delete(
            idServer,
            pref.getString(Constants.USER_TOKEN)
        )
            .enqueue(object : Callback<Void> {
                override fun onResponse(
                    call: Call<Void>,
                    response: Response<Void>
                ) {

                    if (response.isSuccessful) {

                        actionAfterSuccess()
                        showMessage("SharedWishlist removed")
                        changeVisibilityLoadingScreen(false)

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("SharedWishlist", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }

                }

                override fun onFailure(call: Call<Void>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun refreshData() {
        sharedWishlistList.clear()
        sharedWishlistList.addAll(sharedWishlistRepository.getAll())
        adapter.notifyDataSetChanged()
    }

    override fun onResume() {
        getAllSharedWishlistFromServer()
        (requireActivity() as MainActivity).setRefreshMyWishlistData(::getAllSharedWishlistFromServer)
        super.onResume()
    }

    private fun addSharedWishlist() {
        val dialogBuilder = AlertDialog.Builder(requireContext())
        dialogBuilder.setTitle("Adding new shared wishlist")
        dialogBuilder.setMessage("Choose method of inserting")

        dialogBuilder.setPositiveButton("QR") { _, _ ->
            val action =
                SharedWishlistsFragmentDirections.actionNavigationSharedWishlistsToNavigationSharedWishlistsScan()
            findNavController().navigate(action)
        }

        dialogBuilder.setNeutralButton("Cancel") { dialog, _ ->
            dialog.dismiss()
        }

        dialogBuilder.setNegativeButton("Code") { dialog, _ ->
            val action =
                SharedWishlistsFragmentDirections.actionNavigationSharedWishlistsToNavigationSharedWishlistsAdd()
            findNavController().navigate(action)
            dialog.dismiss()
        }

        dialogBuilder.setCancelable(true)

        dialogBuilder.create().show()
    }

    private fun getAllSharedWishlistFromServer() {

        if (!NetworkUtils.isConnected(requireContext())) {
            showMessage("Device is not connected to the internet.")
            refreshData()
            return
        }

        changeVisibilityLoadingScreen(true)

        sharedItemService.getAll(pref.getString(Constants.USER_TOKEN))
            .enqueue(object : Callback<List<SharedWishlist>> {
                override fun onResponse(
                    call: Call<List<SharedWishlist>>,
                    response: Response<List<SharedWishlist>>
                ) {

                    val listOfWishlists: List<SharedWishlist>? = response.body()
                    if (response.isSuccessful && listOfWishlists != null) {

                        sharedWishlistRepository.deleteAll()
                        for (sharedWishlist in listOfWishlists) {
                            sharedWishlistRepository.saveWithItems(sharedWishlist)
                        }

                        refreshData()
                        showMessage("Shared wishlists are in sync.")
                        changeVisibilityLoadingScreen(false)

                    } else {
                        val errorMessage = RetrofitUtil.parseErrorCode("SharedWishlistS", response.code())
                        Log.e(this::class.simpleName, errorMessage)
                        refreshData()
                        changeVisibilityLoadingScreen(false)
                        showMessage(errorMessage)
                    }

                }

                override fun onFailure(call: Call<List<SharedWishlist>>, t: Throwable) {
                    val errorMessage = RetrofitUtil.parseErrorMessage(t)
                    Log.e(this::class.simpleName, errorMessage, t)
                    refreshData()
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            })
    }

    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }
}