package cz.muni.pv239.wishlist.ui.sharedwishlists.scanning

import android.Manifest.permission.CAMERA
import android.content.pm.PackageManager
import android.os.Bundle
import android.service.autofill.OnClickAction
import android.util.Base64
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.navigation.fragment.findNavController
import com.budiyev.android.codescanner.AutoFocusMode
import com.budiyev.android.codescanner.CodeScanner
import com.budiyev.android.codescanner.CodeScannerView
import com.budiyev.android.codescanner.DecodeCallback
import com.budiyev.android.codescanner.ErrorCallback
import com.budiyev.android.codescanner.ScanMode
import com.google.android.material.snackbar.Snackbar
import com.google.gson.Gson
import com.google.gson.JsonSyntaxException

import cz.muni.pv239.wishlist.activities.MainActivity
import cz.muni.pv239.wishlist.databinding.FragmentSharedWishlistsScanBinding
import cz.muni.pv239.wishlist.model.mywishlist.SharingWishlistWrapper
import cz.muni.pv239.wishlist.restclient.RetrofitUtil
import cz.muni.pv239.wishlist.restclient.SharedItemService
import cz.muni.pv239.wishlist.utils.Constants
import cz.muni.pv239.wishlist.utils.SharedPref
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


class ScanFragment : Fragment() {

    private lateinit var binding: FragmentSharedWishlistsScanBinding
    private lateinit var pref: SharedPref
    private lateinit var codeScanner: CodeScanner
    private lateinit var scannerView: CodeScannerView
    private val gson: Gson by lazy {
        Gson()
    }
    private val sharedItemService: SharedItemService by lazy {
        RetrofitUtil.createSharedItemService()
    }


    private val requestPermissionLauncher = registerForActivityResult(
        ActivityResultContracts.RequestPermission()
    ) { isGranted ->
        if (isGranted) {
            setupControls()
        } else {
            showMessage("You need to give permission to app to use camera.")
            findNavController().navigateUp()
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSharedWishlistsScanBinding.inflate(layoutInflater, container, false)
        pref = SharedPref(requireContext())
        scannerView = binding.scannerView

        config()
        initUiElems()

        return binding.root
    }

    private fun initUiElems() {

        binding.tilScannedCode.getInputTextLayout().isEnabled = false
        binding.btSave.setOnClickListener {

            var inputText = binding.tilScannedCode.getInputTextLayout().editText?.text.toString().trim()
            if (inputText == "") {
                showMessage("You must scan code first.")
                return@setOnClickListener
            }

            try {
                val decodedString = String(java.util.Base64.getDecoder().decode(inputText.replace("\n", "")))
                var sharedWishlistRequest = gson.fromJson(decodedString, SharingWishlistWrapper::class.java)

                addSharedWishlist(sharedWishlistRequest)

            } catch (e: IllegalArgumentException) {
                showMessage("Wrong code inserted.")
                return@setOnClickListener
            } catch (e: JsonSyntaxException) {
                showMessage("Wrong code format.")
                return@setOnClickListener
            }
        }
    }

    private fun addSharedWishlist(sharedWishlistRequest: SharingWishlistWrapper) {
        changeVisibilityLoadingScreen(true)

        sharedItemService.addSharedWishlistToUser(
            sharedWishlistRequest,
            pref.getString(Constants.USER_TOKEN)
        ).enqueue(object : Callback<Void> {
            override fun onResponse(
                call: Call<Void>,
                response: Response<Void>
            ) {

                if (response.isSuccessful) {

                    showMessage("Wishlist added.")
                    findNavController().navigateUp()
                    showMessage("SharedWishlist added")
                    changeVisibilityLoadingScreen(false)

                } else {
                    val errorMessage = RetrofitUtil.parseErrorCode("SharedWishlist", response.code())
                    Log.e(this::class.simpleName, errorMessage)
                    changeVisibilityLoadingScreen(false)
                    showMessage(errorMessage)
                }
            }
            override fun onFailure(call: Call<Void>, t: Throwable) {
                val errorMessage = RetrofitUtil.parseErrorMessage(t)
                Log.e(this::class.simpleName, errorMessage, t)
                changeVisibilityLoadingScreen(false)
                showMessage(errorMessage)
            }
        })
    }

    private fun config() {

        if (ContextCompat.checkSelfPermission(requireActivity(), CAMERA) != PackageManager.PERMISSION_GRANTED) {
            askForCameraPermission()
        } else {
            setupControls()
        }
    }

    private fun askForCameraPermission() {
        requestPermissionLauncher.launch(CAMERA)
    }

    fun setupControls() {
        codeScanner = CodeScanner(requireContext(), scannerView)

        codeScanner.camera = CodeScanner.CAMERA_BACK
        codeScanner.formats = CodeScanner.ALL_FORMATS
        codeScanner.autoFocusMode = AutoFocusMode.SAFE
        codeScanner.scanMode = ScanMode.CONTINUOUS
        codeScanner.isAutoFocusEnabled = true
        codeScanner.isFlashEnabled = false

        codeScanner.decodeCallback = DecodeCallback {
            requireActivity().runOnUiThread {
               binding.tilScannedCode.getInputTextLayout().editText?.setText(it.text)
            }
        }
        codeScanner.errorCallback = ErrorCallback { // or ErrorCallback.SUPPRESS
            requireActivity().runOnUiThread {
                showMessage("Camera initialization error: ${it.message}")
            }
        }

        codeScanner.startPreview()
    }


    private fun changeVisibilityLoadingScreen(isVisible: Boolean) {
        (requireActivity() as MainActivity).changeVisibilityLoadingScreen(isVisible)
    }

    private fun showMessage(message: String) {
        Snackbar.make(binding.root, message, Snackbar.LENGTH_SHORT).show()
    }


}